/////////////////////////////////////////////////////////////////////////////
//                                                                         //
//  Dorgem - Webcam Capture application                                    //
//  Copyright (C) 2001-2007 Frank Fesevur                                  //
//                                                                         //
//  This program is free software; you can redistribute it and/or modify   //
//  it under the terms of the GNU General Public License as published by   //
//  the Free Software Foundation; either version 2 of the License, or      //
//  (at your option) any later version.                                    //
//                                                                         //
//  This program is distributed in the hope that it will be useful,        //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of         //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the           //
//  GNU General Public License for more details.                           //
//                                                                         //
//  You should have received a copy of the GNU General Public License      //
//  along with this program; if not, write to the Free Software            //
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.              //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////
// CaptionBitmap.cpp : implementation file

#include "Dorgem.h"
#include "CaptionBitmap.h"

/////////////////////////////////////////////////////////////////////////////
//

CCaptionBitmap::CCaptionBitmap()
{
  m_iType = CAPTION_BMP;
  m_iRotate = 0;
  m_iScalePercentage = 0;
  m_iScaleX = 0;
  m_iScaleY = 0;
}

/////////////////////////////////////////////////////////////////////////////
//

void CCaptionBitmap::Rotate(wxImage& img)
{
  // Do the 90-180-270 with the Rotate90().
  // This is done for speed reasons
  switch (m_iRotate)
  {
    case   0:                                   return;
    case  90: img = img.Rotate90();             return;
    case 180: img = img.Rotate90().Rotate90();  return;
    case 270: img = img.Rotate90(false);        return;
  }

  // All others with the Rotate()
  wxPoint pt(img.GetWidth() / 2, img.GetHeight() / 2);
  img = img.Rotate((double) m_iRotate * (M_PI / 180.0), pt);
}

/////////////////////////////////////////////////////////////////////////////
//

void CCaptionBitmap::Scale(wxImage& img)
{
  // Need to scale to a certain percentage
  if (m_iScalePercentage != 0)
  {
    double dFactor = (double) m_iScalePercentage / 100.0;
    int x = (int) ((double) img.GetWidth() * dFactor);
    int y = (int) ((double) img.GetHeight() * dFactor);
    img = img.Scale(x, y, wxIMAGE_QUALITY_HIGH);
    return;
  }

  // Scale to a fixed width
  if (m_iScaleX != 0 && m_iScaleY != 0)
    img = img.Scale(m_iScaleX, m_iScaleY, wxIMAGE_QUALITY_HIGH);
}

/////////////////////////////////////////////////////////////////////////////
//

bool CCaptionBitmap::Overlay(wxImage& imgCaptured)
{
  // Is the location of the bitmap set?
  if (!m_fnImage.IsOk())
    return(false);

  // Load the bitmap
  wxImage imgFile;
  if (!imgFile.LoadFile(m_fnImage.GetFullPath()))
    return(false);

  // Rotate the overlay
  Rotate(imgFile);

  // Scale the overlay
  Scale(imgFile);

  // Calculate the position
  int iPosX = 0, iPosY = 0;
  if (m_bAbsolute)
  {
    iPosX = m_iPosHor;
    iPosY = m_iPosVer;
  }
  else
  {
    switch (m_iPosHor)
    {
      case POS_LEFT:
        iPosX = 0;
        break;
      case POS_CENTER:
        iPosX = (imgCaptured.GetWidth() - imgFile.GetWidth()) / 2;
        break;
      case POS_RIGHT:
        iPosX = imgCaptured.GetWidth() - imgFile.GetWidth();
        break;
    }
    switch (m_iPosVer)
    {
      case POS_TOP:
        iPosY = 0;
        break;
      case POS_CENTER:
        iPosY = (imgCaptured.GetHeight() - imgFile.GetHeight()) / 2;
        break;
      case POS_BOTTOM:
        iPosY = imgCaptured.GetHeight() - imgFile.GetHeight();
        break;
    }
  }

  // Paste the loaded image to the captured image
  imgCaptured.Paste(imgFile, iPosX, iPosY);
  return(true);
}

/////////////////////////////////////////////////////////////////////////////
//

bool CCaptionBitmap::Read(TiXmlElement* pElem)
{
  if (!CCaptionEvent::Read(pElem))
    return(false);

  m_fnImage = wxGetApp().GetAttributeString(pElem, g_strXmlFile);
  m_iRotate = wxGetApp().GetAttributeInt(pElem, g_strXmlRotate, 0);
  m_iScalePercentage = wxGetApp().GetAttributeInt(pElem, g_strXmlScalePercentage, 0);
  m_iScaleX = wxGetApp().GetAttributeInt(pElem, g_strXmlScaleX, 0);
  m_iScaleY = wxGetApp().GetAttributeInt(pElem, g_strXmlScaleY, 0);

  return(true);
}

/////////////////////////////////////////////////////////////////////////////
//

bool CCaptionBitmap::Write(TiXmlElement* pElem)
{
  TiXmlElement elemCaption(g_strXmlCaption);
  elemCaption.SetAttribute(g_strXmlType, g_strXmlBitmap);

  if (!CCaptionEvent::Write(&elemCaption))
    return(false);

  elemCaption.SetAttribute(g_strXmlFile, m_fnImage.GetFullPath());
  elemCaption.SetAttribute(g_strXmlRotate, m_iRotate);
  elemCaption.SetAttribute(g_strXmlScalePercentage, m_iScalePercentage);
  elemCaption.SetAttribute(g_strXmlScaleX, m_iScaleX);
  elemCaption.SetAttribute(g_strXmlScaleY, m_iScaleY);

  pElem->InsertEndChild(elemCaption);
  return(true);
}
