/////////////////////////////////////////////////////////////////////////////
//                                                                         //
//  Dorgem - Webcam Capture application                                    //
//  Copyright (C) 2001-2007 Frank Fesevur                                  //
//                                                                         //
//  This program is free software; you can redistribute it and/or modify   //
//  it under the terms of the GNU General Public License as published by   //
//  the Free Software Foundation; either version 2 of the License, or      //
//  (at your option) any later version.                                    //
//                                                                         //
//  This program is distributed in the hope that it will be useful,        //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of         //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the           //
//  GNU General Public License for more details.                           //
//                                                                         //
//  You should have received a copy of the GNU General Public License      //
//  along with this program; if not, write to the Free Software            //
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.              //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////
// CaptionManager.cpp: implementation of the CCaptionManager class.

#include "Dorgem.h"
#include "DorgemEvent.h"
#include "CaptionManager.h"
#include "CaptionEvent.h"
#include "CaptionText.h"
#include "CaptionBitmap.h"

/////////////////////////////////////////////////////////////////////////////
// Read the events from the registry

bool CCaptionManager::ReadEvents(TiXmlElement* pElemStore)
{
  wxLogDebug("[CCaptionManager]ReadEvents()");

  bool bRet = false;

  // Go through the child node to get the caption events
  TiXmlNode* pNode = pElemStore->FirstChildElement();
  while (pNode != NULL)
  {
    wxString strElementName = pNode->Value();

    if (strElementName == g_strXmlCaption)
    {
      TiXmlElement* pElemCaption = pNode->ToElement();
      wxASSERT(pElemCaption != NULL);

      wxString strType = pElemCaption->Attribute(g_strXmlType);
      CCaptionEvent* pEvent = NULL;

      // Go through the supported types
      if (strType.CmpNoCase(g_strXmlBitmap) == 0)
        pEvent = new CCaptionBitmap;
      else if (strType.CmpNoCase(g_strXmlText) == 0)
        pEvent = new CCaptionText;

      // Get the information of that class and add it to the list
      if (pEvent != NULL)
        if (pEvent->Read(pElemCaption))
          if (AddEvent(pEvent))
            bRet = true;
    }

    pNode = pElemStore->IterateChildren(pNode);
  }

  return(bRet);
}

/////////////////////////////////////////////////////////////////////////////
// Write the events to the registry

bool CCaptionManager::WriteEvents(TiXmlElement* pElemStore)
{
  wxLogDebug("[CCaptionManager]WriteEvents()");

  Order();
  for (size_t i = 0; i < m_pEvent.GetCount(); i++)
    m_pEvent[i]->Write(pElemStore);

  return(true);
}
