/////////////////////////////////////////////////////////////////////////////
//                                                                         //
//  Dorgem - Webcam Capture application                                    //
//  Copyright (C) 2001-2006 Frank Fesevur                                  //
//                                                                         //
//  This program is free software; you can redistribute it and/or modify   //
//  it under the terms of the GNU General Public License as published by   //
//  the Free Software Foundation; either version 2 of the License, or      //
//  (at your option) any later version.                                    //
//                                                                         //
//  This program is distributed in the hope that it will be useful,        //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of         //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the           //
//  GNU General Public License for more details.                           //
//                                                                         //
//  You should have received a copy of the GNU General Public License      //
//  along with this program; if not, write to the Free Software            //
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.              //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////
// CaptionText.cpp : implementation file

#include "Dorgem.h"
#include "CaptionText.h"

#include <wx/textfile.h>

/////////////////////////////////////////////////////////////////////////////
//

CCaptionText::CCaptionText()
{
  m_iType = CAPTION_TXT;
  m_bAbsolute = false;
  m_uMaxLength = 0;
  m_uRotate = 0;
  m_bTransparent = true;
  m_uShadow = SHADOW_FULL;
  m_colBack.Set(DEFAULT_BACK_COLOR);
  m_colFore.Set(DEFAULT_FORE_COLOR);

  // The default font is platform specific
#ifdef __WXMSW__
  SetFont(wxT("Verdana"));
#endif
}

/////////////////////////////////////////////////////////////////////////////
//

void CCaptionText::SetFont(wxString str, int iSize, int iWeight, int iStyle, bool bUnderlined)
{
  m_strFontName = str;
  m_iFontSize = iSize;
  m_iFontWeight = iWeight;
  m_iFontStyle = iStyle;
  m_bFontUnderlined = bUnderlined;
}

/////////////////////////////////////////////////////////////////////////////
// Put the caption text on the bitmap

bool CCaptionText::Overlay(wxImage& image)
{
  wxString strText;
  if (m_strFilename.IsEmpty())
  {
    // Construct the time related text
    strText = wxGetApp().DateFormat(m_strCaption);
  }
  else
  {
    // Read the first line of the text file
    wxTextFile file(m_strFilename);
    if (file.Exists())
    {
      if (file.Open())
      {
        strText = file.GetFirstLine();
        file.Close();
      }
    }
  }

  // Is the overlay text set?
  if (strText.IsEmpty())
    return(false);

  // Maximize the length of the text
  if (m_uMaxLength != 0)
    strText = strText.Left(m_uMaxLength);

  // Create a MemoryDC with the wxImage to add the text to
  wxBitmap bmp(image);
  wxMemoryDC dc(bmp);

  // Create the font
  dc.SetFont(wxFont(m_iFontSize, wxFONTFAMILY_UNKNOWN, m_iFontStyle, m_iFontWeight, m_bFontUnderlined, m_strFontName));

  // Calculate the position
  wxSize sizeText = dc.GetTextExtent(strText);
  int iPosX = 0, iPosY = 0;
  if (m_bAbsolute)
  {
    iPosX = m_iPosHor;
    iPosY = m_iPosVer;
  }
  else
  {
    switch (m_iPosHor)
    {
      case POS_LEFT:
        iPosX = 0;
        break;
      case POS_CENTER:
        iPosX = (image.GetWidth() - sizeText.x) / 2;
        break;
      case POS_RIGHT:
        iPosX = image.GetWidth() - sizeText.x;
        break;
    }
    switch (m_iPosVer)
    {
      case POS_TOP:
        iPosY = 0;
        break;
      case POS_CENTER:
        iPosY = (image.GetHeight() - sizeText.y) / 2;
        break;
      case POS_BOTTOM:
        iPosY = image.GetHeight() - sizeText.y - 1;  // Extra margin of 1 pixel for digits like 'g'
        break;
    }
  }

  // Set the colors
  if (m_bTransparent)
  {
    dc.SetBackgroundMode(wxTRANSPARENT);
    dc.SetTextForeground(m_colBack);

    switch (m_uShadow)
    {
      case SHADOW_FULL:
      {
        int i, j, iArr[3] = { -1, 0, 1 };
        for (i = 0; i < 3; i++)
          for (j = 0; j < 3; j++)
            dc.DrawRotatedText(strText, iPosX + iArr[i], iPosY + iArr[j], m_uRotate);
        break;
      }
      case SHADOW_DROP:
        dc.DrawRotatedText(strText, iPosX + 1, iPosY + 1, m_uRotate);
        break;
    }
  }
  else
  {
    dc.SetBackgroundMode(wxSOLID);
    dc.SetTextBackground(m_colBack);
  }

  // Finally draw the real text
  dc.SetTextForeground(m_colFore);
  dc.DrawRotatedText(strText, iPosX, iPosY, m_uRotate);

  // Put the new created image (in the MemoryDC) back in the wxImage
  image = bmp.ConvertToImage();
  return(true);
}

/////////////////////////////////////////////////////////////////////////////
//

bool CCaptionText::Read(TiXmlElement* pElem)
{
  if (!CCaptionEvent::Read(pElem))
    return(false);

  m_strCaption = wxGetApp().GetAttributeString(pElem, g_strXmlCaption);
  m_bTransparent  = wxGetApp().GetAttributeBool(pElem, g_strXmlTransparent, false);
  m_uShadow = wxGetApp().GetAttributeInt(pElem, g_strXmlShadow, SHADOW_NONE);
  m_uRotate = wxGetApp().GetAttributeInt(pElem, g_strXmlRotate, 0);
  m_uMaxLength = wxGetApp().GetAttributeInt(pElem, g_strXmlMaxLength, 0);
  m_strFilename = wxGetApp().GetAttributeString(pElem, g_strXmlFilename);
  m_colBack = wxGetApp().GetAttributeColour(pElem, g_strXmlBackColor, DEFAULT_BACK_COLOR);
  m_colFore = wxGetApp().GetAttributeColour(pElem, g_strXmlForeColor, DEFAULT_FORE_COLOR);
  m_strFontName = wxGetApp().GetAttributeString(pElem, g_strXmlFontName);
  //elemCaption.SetAttribute(g_strXmlFontSize, m_iFontSize);
  //elemCaption.SetAttribute(g_strXmlFontWeight, m_iFontWeight);
  //elemCaption.SetAttribute(g_strXmlFontStyle, m_iFontStyle);
  //elemCaption.SetAttribute(g_strXmlFontUnderlined, m_bFontUnderlined);

  return(true);
}

/////////////////////////////////////////////////////////////////////////////
//

bool CCaptionText::Write(TiXmlElement* pElem)
{
  TiXmlElement elemCaption(g_strXmlCaption);
  elemCaption.SetAttribute(g_strXmlType, g_strXmlText);

  if (!CCaptionEvent::Write(&elemCaption))
    return(false);

  elemCaption.SetAttribute(g_strXmlCaption, m_strCaption);
  elemCaption.SetAttribute(g_strXmlTransparent, m_bTransparent);
  elemCaption.SetAttribute(g_strXmlShadow, m_uShadow);
  //elemCaption.SetAttribute(g_strXmlRotate, m_uRotate);
  elemCaption.SetAttribute(g_strXmlMaxLength, m_uMaxLength);
  elemCaption.SetAttribute(g_strXmlFilename, m_strFilename);
  elemCaption.SetAttribute(g_strXmlBackColor, m_colBack.GetAsString(wxC2S_HTML_SYNTAX));
  elemCaption.SetAttribute(g_strXmlForeColor, m_colFore.GetAsString(wxC2S_HTML_SYNTAX));
  elemCaption.SetAttribute(g_strXmlFontName, m_strFontName);
  elemCaption.SetAttribute(g_strXmlFontSize, m_iFontSize);
  elemCaption.SetAttribute(g_strXmlFontWeight, m_iFontWeight);
  elemCaption.SetAttribute(g_strXmlFontStyle, m_iFontStyle);
  elemCaption.SetAttribute(g_strXmlFontUnderlined, m_bFontUnderlined);

  pElem->InsertEndChild(elemCaption);
  return(true);
}
