/////////////////////////////////////////////////////////////////////////////
//                                                                         //
//  Dorgem - Webcam Capture application                                    //
//  Copyright (C) 2001-2007 Frank Fesevur                                  //
//                                                                         //
//  This program is free software; you can redistribute it and/or modify   //
//  it under the terms of the GNU General Public License as published by   //
//  the Free Software Foundation; either version 2 of the License, or      //
//  (at your option) any later version.                                    //
//                                                                         //
//  This program is distributed in the hope that it will be useful,        //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of         //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the           //
//  GNU General Public License for more details.                           //
//                                                                         //
//  You should have received a copy of the GNU General Public License      //
//  along with this program; if not, write to the Free Software            //
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.              //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////
// CaptionText.h : header file

#ifndef __CAPTIONTEXT_H__
#define __CAPTIONTEXT_H__

#include "CaptionEvent.h"

#define DEFAULT_FORE_COLOR ((long) 0xffffff)
#define DEFAULT_BACK_COLOR ((long) 0x000000)

#define SHADOW_NONE   0
#define SHADOW_FULL   1
#define SHADOW_DROP   2

/////////////////////////////////////////////////////////////////////////////
//

class CCaptionText : public CCaptionEvent
{
public:
  CCaptionText();

  bool Overlay(wxImage&);

  wxString  GetCaption()        { return(m_strCaption);   };
  wxString  GetFilename()       { return(m_strFilename);  };
  bool      GetTranparent()     { return(m_bTransparent); };
  UINT      GetShadow()         { return(m_uShadow);      };
  UINT      GetRotate()         { return(m_uRotate);      };
  UINT      GetMaxLength()      { return(m_uMaxLength);   };
  wxColour  GetBackColor()      { return(m_colBack);      };
  wxColour  GetForeColor()      { return(m_colFore);      };

  void SetCaption(wxString s)   { m_strCaption = s;       };
  void SetFilename(wxString s)  { m_strFilename = s;      };
  void SetTranparent(bool b)    { m_bTransparent = b;     };
  void SetShadow(UINT u)        { m_uShadow = u;          };
  void SetRotate(UINT u)        { m_uRotate = u;          };
  void SetMaxLength(UINT u)     { m_uMaxLength = u;       };
  void SetBackColor(wxColour c) { m_colBack = c;          };
  void SetForeColor(wxColour c) { m_colFore = c;          };
  void SetFont(wxString str, int iSize = 10, int iWeight = wxFONTWEIGHT_NORMAL, int iStyle = wxFONTSTYLE_NORMAL, bool bUnderlined = false);

  bool Read(TiXmlElement*);
  bool Write(TiXmlElement*);

protected:
  wxString  m_strCaption;       // The caption text (can contain % tags)
  bool      m_bTransparent;     // Is the text drawn transparent
  UINT      m_uShadow;          // The type of shadow
  UINT      m_uRotate;          // The rotation angle of the text (in whole degrees)
  UINT      m_uMaxLength;       // The maximun length of the text (0 = no limit)
  wxString  m_strFilename;      // Filename of the text file to read the caption from
  wxColour  m_colBack;          // The background color
  wxColour  m_colFore;          // The text color

  wxString  m_strFontName;      // The name of the font
  int       m_iFontSize;        // Font size in points
  int       m_iFontWeight;      // Font weight
  int       m_iFontStyle;       // Font style (Normal or Italic)
  bool      m_bFontUnderlined;  // Font underlined

};

#endif // __CAPTIONTEXT_H__
