/////////////////////////////////////////////////////////////////////////////
//                                                                         //
//  Dorgem - Webcam Capture application                                    //
//  Copyright (C) 2001-2007 Frank Fesevur                                  //
//                                                                         //
//  This program is free software; you can redistribute it and/or modify   //
//  it under the terms of the GNU General Public License as published by   //
//  the Free Software Foundation; either version 2 of the License, or      //
//  (at your option) any later version.                                    //
//                                                                         //
//  This program is distributed in the hope that it will be useful,        //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of         //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the           //
//  GNU General Public License for more details.                           //
//                                                                         //
//  You should have received a copy of the GNU General Public License      //
//  along with this program; if not, write to the Free Software            //
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.              //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////

#ifndef __DORGEM_H__
#define __DORGEM_H__

// For compilers that support precompilation, includes "wx/wx.h".
#include <wx/wxprec.h>

// for all others, include the necessary headers (this file is usually all you
// need because it includes almost all "standard" wxWindows headers)
#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif

#include <wx/snglinst.h>
#include <wx/statline.h>
#include <wx/statbox.h>
#include <wx/dcclient.h>
#include <wx/image.h>
#include <wx/datetime.h>
#include <wx/filename.h>
#include <wx/mstream.h>

class TiXmlElement;
class CProfileManager;

/////////////////////////////////////////////////////////////////////////////
// The application

class CDorgemApp : public wxApp
{
public:
  CDorgemApp();
  ~CDorgemApp();

  virtual bool OnInit();
  virtual int  OnExit();

  wxString DateFormat(wxString str);

  wxString          m_strAppPath;
  wxFileName        m_fnSettingsPath;
  CProfileManager*  m_pProfileMngr;

  int       GetAttributeInt(TiXmlElement* pElem, wxString strAttr, int iDefault);
  bool      GetAttributeBool(TiXmlElement* pElem, wxString strAttr, bool bDefault);
  wxString  GetAttributeString(TiXmlElement* pElem, wxString strAttr, wxString strDefault = wxT(""));
  wxColour  GetAttributeColour(TiXmlElement* pElem, wxString strAttr, wxColour colDefault);
  void      Crypt(wxString& strMessage);
  void      DeCrypt(wxString& strMessage);

private:
  wxSingleInstanceChecker* m_pChecker;
  wxLocale    m_locale;                   // locale we'll be using

  void        GetXmlSettingsPath();
  bool        ReadSettingsFromXml();
  bool        WriteSettingsToXml();

  // Keep as last!
  DECLARE_EVENT_TABLE();

};

DECLARE_APP(CDorgemApp);

#include "Version.h"

// Useful macros for easy access to the different types of standard message boxes
#define MsgBoxEx(s)           wxMessageBox(s, wxT(PRODUCT_NAME), wxOK | wxICON_EXCLAMATION | wxCENTER, this)
#define MsgBoxError(s)        wxMessageBox(s, wxT(PRODUCT_NAME), wxOK | wxICON_ERROR | wxCENTER, this)
#define MsgBoxInfo(s)         wxMessageBox(s, wxT(PRODUCT_NAME), wxOK | wxICON_INFORMATION | wxCENTER, this)
#define MsgBoxYesNo(s)       (wxMessageBox(s, wxT(PRODUCT_NAME), wxYES_NO | wxICON_QUESTION | wxCENTER) == wxYES, this)
#define MsgBoxYesNoCancel(s)  wxMessageBox(s, wxT(PRODUCT_NAME), wxYES_NO | wxCANCEL | wxICON_QUESTION | wxCENTER, this)

#endif // __BLACKBOX_H__
