/////////////////////////////////////////////////////////////////////////////
//                                                                         //
//  Dorgem - Webcam Capture application                                    //
//  Copyright (C) 2001-2007 Frank Fesevur                                  //
//                                                                         //
//  This program is free software; you can redistribute it and/or modify   //
//  it under the terms of the GNU General Public License as published by   //
//  the Free Software Foundation; either version 2 of the License, or      //
//  (at your option) any later version.                                    //
//                                                                         //
//  This program is distributed in the hope that it will be useful,        //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of         //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the           //
//  GNU General Public License for more details.                           //
//                                                                         //
//  You should have received a copy of the GNU General Public License      //
//  along with this program; if not, write to the Free Software            //
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.              //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////

#include "Dorgem.h"
#include "DorgemDlg.h"
#include "ScrollTextDlg.h"
#include "VideoWindow.h"
#include "StoreFile.h"
#include "ProfileManager.h"
#include "WebServer.h"

#include "../TinyXml/tinyxml.h"

#include <wx/aboutdlg.h>
#include <wx/url.h>
#include <wx/stream.h>

///////////////////////////////////////////////////////////////////////////
//

BEGIN_EVENT_TABLE(CDorgemDlg, wxFrame)
  EVT_MENU(IDM_FILE_EXIT, CDorgemDlg::OnFileExit)
  EVT_MENU(IDM_FILE_TEST, CDorgemDlg::OnFileTest)
  EVT_MENU(IDM_HELP_LICENSE, CDorgemDlg::OnHelpLicense)
  EVT_MENU(IDM_HELP_CHECK_VERSION, CDorgemDlg::OnHelpCheckVersion)
  EVT_MENU(IDM_HELP_ABOUT, CDorgemDlg::OnHelpAbout)
  EVT_TIMER(ID_TIMER, CDorgemDlg::OnTimer)
END_EVENT_TABLE()

/////////////////////////////////////////////////////////////////////////////
//

CDorgemDlg::CDorgemDlg(wxWindow* pParent) :
  wxFrame(pParent, -1, wxT(PRODUCT_NAME), wxPoint(-1, -1), wxSize(510, 500), wxDEFAULT_DIALOG_STYLE | wxMINIMIZE_BOX)
{
  // Set the frame icon
  SetIcon(wxICON(IDI_APPLICATION));

  // Create a 'File' menu
  wxMenu *menuFile = new wxMenu;
  menuFile->Append(IDM_FILE_TEST, "&Test\tCtrl+T");
  menuFile->AppendSeparator();
  menuFile->Append(IDM_FILE_EXIT, _("E&xit"));

  // Create a 'Help' menu
  wxMenu *menuHelp = new wxMenu;
  menuHelp->Append(IDM_HELP_LICENSE, wxString(_("&License...")) + wxT("\tCtrl+L"));
  menuHelp->Append(IDM_HELP_CHECK_VERSION, wxString(_("&Check for new version...")));
  menuHelp->AppendSeparator();
  menuHelp->Append(IDM_HELP_ABOUT, wxString(_("&About...")) + wxT("\tF1"));

  // Now append the freshly created menus to the menu bar...
  wxMenuBar *menuBar = new wxMenuBar();
  menuBar->Append(menuFile, _("&File"));
  menuBar->Append(menuHelp, _("&Help"));

  // ... and attach this menu bar to the frame
  SetMenuBar(menuBar);

  // Make it look like a dialog
  SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE));

  //
  m_logTextCtrl = new wxTextCtrl(this, -1, wxT("Event Log for wxVideoCaptureWindow \n"), wxPoint(0,0), wxSize(500,100), wxTE_MULTILINE );
  m_vidCapWin = new CVideoWindow(this , ID_VIDEOWIN, wxPoint(0,105), wxSize(400,300));

  wxLogDebug("Number of devices: %d", m_vidCapWin->GetDeviceCount());
  bool bShowCapWin = false;
  if (m_vidCapWin->GetDeviceCount() > 0)
  {
    for (int i = 0; i < m_vidCapWin->GetDeviceCount(); i++)
      wxLogDebug("Devices(%d): %s %s", i, m_vidCapWin->GetDeviceName(i), m_vidCapWin->GetDeviceVersion(i));

    if (m_vidCapWin->DeviceConnect(0))
    {
      int iWidth, iHeight, iBPP;
      FOURCC fourcc;

      m_vidCapWin->GetVideoFormat(&iWidth, &iHeight, &iBPP, &fourcc);
      m_vidCapWin->SetSize(wxSize(iWidth + 2, iHeight + 2));   // Add 2 for the borders (dirty)
      m_vidCapWin->Preview(true);
      bShowCapWin = true;
    }
  }
  m_vidCapWin->Show(bShowCapWin);

  // Set the timer to 1 second
  m_timer.SetOwner(this, ID_TIMER);
  m_timer.Start(1000);

  // Start the web server
  //m_pServer = NULL;
  m_pServer = new CWebServer(this, m_vidCapWin);
  m_pServer->Start();
}

/////////////////////////////////////////////////////////////////////////////
//

CDorgemDlg::~CDorgemDlg()
{
  if (m_pServer != NULL)
    delete m_pServer;
}

/////////////////////////////////////////////////////////////////////////////
//

void CDorgemDlg::OnFileExit(wxCommandEvent& WXUNUSED(event))
{
  m_vidCapWin->Preview(false);    // allows for quicker closing
  Close(TRUE);                    // TRUE is to force the frame to close
}

/////////////////////////////////////////////////////////////////////////////
//

void CDorgemDlg::OnHelpLicense(wxCommandEvent& WXUNUSED(event))
{
  CScrollTextDlg dlg(this, wxSize(350, 200),
                      _("Dorgem is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License (GPL) which is included in the original archive.\n"
                        "\n"
                        "NO WARRANTY - have fun!"));
  dlg.ShowModal();
}

/////////////////////////////////////////////////////////////////////////////
//

void CDorgemDlg::OnHelpCheckVersion(wxCommandEvent& WXUNUSED(event))
{
#ifdef __WXDEBUG__
  wxURL url(wxT("http://localhost/version.txt"));
#else
  wxURL url(wxT("http://dorgem.sourceforge.net/version.txt"));
#endif

  if (url.GetError() != wxURL_NOERR)
  {
    wxLogDebug(wxT("Error getting version.txt"));
    return;
  }

  wxInputStream* in = url.GetInputStream();
  if (in == NULL)
  {
    wxLogDebug(wxT("Error opening input stream for version.txt"));
    return;
  }

  size_t size = in->GetSize();
  wxChar* data = new wxChar[size + 2];
  if (!in->Read(data, size))
  {
    wxLogError(wxT("Read error"));
    return;
  }
  else
  {
    data[size] = 0;
    wxString strNewVersion = data;
    wxLogDebug(wxT("Latest version = '%s'"), strNewVersion);
  }

  delete [] data;
  delete in;

/*
  // Spilt the new version in the seperate numbers
  int iNew[4];
  int iRead = sscanf(strNewVersion, "%d.%d.%d.%d", &iNew[0], &iNew[1], &iNew[2], &iNew[3]);

  // Set the items not read to 0
  for (int i = iRead; i < 4; i++)
    iNew[i] = 0;

  // Check it with the current version number
  CString strMsg;
  int iCurrent[4] = { PRODUCT_VERSION_NUM };
  for (i = 0; i < 4; i++)
  {
    if (iNew[i] > iCurrent[i])
    {
      if (strNewName.IsEmpty())
        strMsg.Format("%s version %s is available!", wxT(PRODUCT_NAME), strNewVersion);
      else
        strMsg.Format("%s version %s is available!", wxT(PRODUCT_NAME), strNewName);
      break;
    }
    if (iNew[i] < iCurrent[i])
      break;
  }

  // If there is no new version, then what?
  if (strMsg.IsEmpty())
  {
    if (strError.IsEmpty())
      strMsg = "You are running the latest version";
    else
      strMsg.Format("Error while checking for newest version:\n%s", strError);
  }
  MsgBoxEx(strMsg);
*/
}

/////////////////////////////////////////////////////////////////////////////
//

void CDorgemDlg::OnHelpAbout(wxCommandEvent& WXUNUSED(event))
{
  wxAboutDialogInfo info;
  info.SetName(wxT(PRODUCT_NAME));
  info.SetVersion(wxT(PRODUCT_VERSION_STR));
  info.SetDescription(wxString::Format(_("Using %s and %s"), wxVERSION_STRING,
                                    wxString::Format(wxT("TinyXML %d.%d.%d"),
                                                      TIXML_MAJOR_VERSION,
                                                      TIXML_MINOR_VERSION,
                                                      TIXML_PATCH_VERSION)));
  info.SetCopyright(wxT(COPYRIGHT_STR));
  info.SetWebSite(wxT("http://dorgem.sourceforge.net"), _("The Dorgem Website"));

  wxAboutBox(info);
}

/////////////////////////////////////////////////////////////////////////////
//

void CDorgemDlg::OnFileTest(wxCommandEvent& WXUNUSED(event))
{
  StoreProfiles(true);
}

/////////////////////////////////////////////////////////////////////////////
//

void CDorgemDlg::OnTimer(wxTimerEvent& WXUNUSED(event))
{
  StoreProfiles(false);
}

/////////////////////////////////////////////////////////////////////////////
//

void CDorgemDlg::StoreProfiles(bool bAlways)
{
  // If the device is not connected, do nothing
  if (!m_vidCapWin->IsDeviceConnected())
    return;

  // Get the image from the camera
  wxImage image;
  CProfileManager* Mngr = wxGetApp().m_pProfileMngr;
  CStoreEvent* pEvent = (CStoreEvent*) Mngr->GetFirstEvent();
  while (pEvent != NULL)
  {
    if (bAlways || pEvent->NeedStore())
    {
      wxLogDebug("[CDorgemDlg]OnFileTest: '%s' Need Storage", pEvent->GetName());

      if (!image.IsOk())
        m_vidCapWin->SnapshotTowxImage(image);

      pEvent->SetImage(image);
      pEvent->Store();
    }

    pEvent = (CStoreEvent*) Mngr->GetNextEvent();
  }
}
