/////////////////////////////////////////////////////////////////////////////
//                                                                         //
//  Dorgem - Webcam Capture application                                    //
//  Copyright (C) 2001-2007 Frank Fesevur                                  //
//                                                                         //
//  This program is free software; you can redistribute it and/or modify   //
//  it under the terms of the GNU General Public License as published by   //
//  the Free Software Foundation; either version 2 of the License, or      //
//  (at your option) any later version.                                    //
//                                                                         //
//  This program is distributed in the hope that it will be useful,        //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of         //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the           //
//  GNU General Public License for more details.                           //
//                                                                         //
//  You should have received a copy of the GNU General Public License      //
//  along with this program; if not, write to the Free Software            //
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.              //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////
// EventManager.cpp: implementation of the CEventManager class.

#include "Dorgem.h"
#include "EventManager.h"

#include "DorgemEvent.h"

#include <wx/arrimpl.cpp>
WX_DEFINE_OBJARRAY(CDorgemEventArray);

/////////////////////////////////////////////////////////////////////////////
// Construction/Destruction

CEventManager::CEventManager()
{
  m_iCount = 0;
}

CEventManager::~CEventManager()
{
  RemoveAll();
}

/////////////////////////////////////////////////////////////////////////////
// Get the first available event

CDorgemEvent* CEventManager::GetFirstEvent()
{
  m_iCount = 0;
  return(GetNextEvent());
}

/////////////////////////////////////////////////////////////////////////////
// Get the next available event. If there are no more events false

CDorgemEvent* CEventManager::GetNextEvent()
{
  if (m_iCount >= m_pEvent.GetCount())
    return(NULL);
  return(m_pEvent[m_iCount++]);
}

/////////////////////////////////////////////////////////////////////////////
//

bool CEventManager::Exist(wxString strEvent)
{
  for (size_t i = 0; i < m_pEvent.GetCount(); i++)
    if (m_pEvent[i]->GetName() == strEvent)
      return(true);

  return(false);
}

/////////////////////////////////////////////////////////////////////////////
//

bool CEventManager::Exist(CDorgemEvent* pEvent)
{
  for (size_t i = 0; i < m_pEvent.GetCount(); i++)
    if (*(m_pEvent[i]) == *pEvent)
      return(true);

  return(false);
}

/////////////////////////////////////////////////////////////////////////////
// Add the event to the array

bool CEventManager::AddEvent(CDorgemEvent* pEvent)
{
  wxASSERT(pEvent->IsValid());

  // Check to see if an event with this name already exists?
  if (Exist(pEvent))
    return(false);

  // Add the event to the list
  m_pEvent.Add(pEvent);
  return(true);
}

/////////////////////////////////////////////////////////////////////////////
// Add the event to the array

bool CEventManager::AddEvent(CDorgemEvent& event)
{
  return(AddEvent(&event));
}

/////////////////////////////////////////////////////////////////////////////
// Clean up the array

void CEventManager::RemoveAll()
{
  size_t i;
  for (i = 0; i < m_pEvent.GetCount(); i++)  
    delete m_pEvent[i];

  m_pEvent.Clear();
}

/////////////////////////////////////////////////////////////////////////////
//

bool CEventManager::Remove(CDorgemEvent* pEvent)
{
  size_t i;
  for (i = 0; i < m_pEvent.GetCount(); i++)
  {
    wxLogDebug("Event = '%s'\n", m_pEvent[i]->GetName());
    if (*(m_pEvent[i]) == *pEvent)
    {
      delete m_pEvent[i];
      m_pEvent.RemoveAt(i);
      return(true);
    }
  }
  return(false);
}

/////////////////////////////////////////////////////////////////////////////
//

bool CEventManager::Remove(wxString strName)
{
  for (size_t i = 0; i < m_pEvent.GetCount(); i++)
  {
    wxLogDebug("Event = '%s'\n", m_pEvent[i]->GetName());
    if (m_pEvent[i]->GetName() == strName)
    {
      m_pEvent.RemoveAt(i);
      return(true);
    }
  }
  return(false);
}

/////////////////////////////////////////////////////////////////////////////
//

/*
//static int wxCMPFUNC_CONV sort_callback(CDorgemEvent **a, CDorgemEvent **b)
static int wxCMPFUNC_CONV sort_callback(CDorgemEvent *a, CDorgemEvent *b)
{
  //return((*a)->GetOrder() > (*b)->GetOrder());
  return(a->GetOrder() > b->GetOrder());
}
*/

/////////////////////////////////////////////////////////////////////////////
//

void CEventManager::Order()
{
/*
  m_pEvent.Sort(sort_callback);

  // Renumber the order numbers
  for (size_t i = 0; i < m_pEvent.GetCount(); i++)
    m_pEvent[i].SetOrder(i + 1);
*/
}
