/////////////////////////////////////////////////////////////////////////////
//                                                                         //
//  Dorgem - Webcam Capture application                                    //
//  Copyright (C) 2001-2007 Frank Fesevur                                  //
//                                                                         //
//  This program is free software; you can redistribute it and/or modify   //
//  it under the terms of the GNU General Public License as published by   //
//  the Free Software Foundation; either version 2 of the License, or      //
//  (at your option) any later version.                                    //
//                                                                         //
//  This program is distributed in the hope that it will be useful,        //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of         //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the           //
//  GNU General Public License for more details.                           //
//                                                                         //
//  You should have received a copy of the GNU General Public License      //
//  along with this program; if not, write to the Free Software            //
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.              //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////
// ProfileManager.cpp: implementation of the CProfileManager class.

#include "Dorgem.h"
#include "ProfileManager.h"

#include "StoreEvent.h"
#include "StoreFile.h"
#include "StoreFTP.h"

/////////////////////////////////////////////////////////////////////////////
// Read the events from the registry

bool CProfileManager::ReadEvents()
{
  wxLogDebug("[CProfileManager]ReadEvents()");

  wxFileName fn = wxGetApp().m_fnSettingsPath;
  fn.SetFullName("Profiles.xml");

  TiXmlDocument doc(fn.GetFullPath());
  if (!doc.LoadFile())
  {
    // If file not found, the initial defaults must be used
    if (doc.ErrorId() == TiXmlBase::TIXML_ERROR_OPENING_FILE)
      return(true);
    else
    {
      wxLogDebug("Could not load file 'Profiles.xml'. Error='%s'. Exiting.", doc.ErrorDesc());
      return(false);
    }
  }

  // Get the "Profiles" element.
  TiXmlNode* nodeProfiles = doc.FirstChild(g_strXmlProfiles);
  if (nodeProfiles == NULL)
    return(false);

  // Go through the available profiles
  TiXmlNode* pNodeProfile = nodeProfiles->FirstChildElement(g_strXmlProfile);
  while (pNodeProfile != NULL)
  {
    TiXmlElement* pElemProfile = pNodeProfile->ToElement();
    wxASSERT(pElemProfile != NULL);

    wxString strType = pElemProfile->Attribute(g_strXmlType);
    CStoreEvent* pEvent = NULL;

    // Go through the supported types
    if (strType.CmpNoCase(g_strXmlFile) == 0)
      pEvent = new CStoreFile;

    if (strType.CmpNoCase(g_strXmlFTP) == 0)
      pEvent = new CStoreFTP;

    // Get the information of that class and add it to the list
    if (pEvent != NULL)
      if (pEvent->Read(pElemProfile))
        AddEvent(pEvent);

    pNodeProfile = nodeProfiles->IterateChildren(pNodeProfile);
  }
  Order();

  return(true);
}

/////////////////////////////////////////////////////////////////////////////
// Write the events to the registry

bool CProfileManager::WriteEvents()
{
  wxLogDebug("[CProfileManager]WriteEvents()");

  wxFileName fn = wxGetApp().m_fnSettingsPath;
  fn.SetFullName("Profiles.xml");

  TiXmlDocument doc(fn.GetFullPath());
  TiXmlElement nodeProfiles(g_strXmlProfiles);
  nodeProfiles.SetAttribute(g_strXmlVersion, PROFILES_XML);

  Order();
  for (size_t i = 0; i < m_pEvent.GetCount(); i++)
    m_pEvent[i]->Write(&nodeProfiles);

  doc.InsertEndChild(nodeProfiles);
  return(doc.SaveFile());
}
