/////////////////////////////////////////////////////////////////////////////
//                                                                         //
//  Dorgem - Webcam Capture application                                    //
//  Copyright (C) 2001-2007 Frank Fesevur                                  //
//                                                                         //
//  This program is free software; you can redistribute it and/or modify   //
//  it under the terms of the GNU General Public License as published by   //
//  the Free Software Foundation; either version 2 of the License, or      //
//  (at your option) any later version.                                    //
//                                                                         //
//  This program is distributed in the hope that it will be useful,        //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of         //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the           //
//  GNU General Public License for more details.                           //
//                                                                         //
//  You should have received a copy of the GNU General Public License      //
//  along with this program; if not, write to the Free Software            //
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.              //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////
// StoreEvent.h : header file

#ifndef __STOREEVENT_H__
#define __STOREEVENT_H__

#define STORE_NONE    0
#define STORE_FILE    1
#define STORE_FTP     2
#define STORE_EMAIL   3
#define STORE_PROGRAM 4
#define STORE_HTTP    5
#define STORE_SERVER  6  // Special case, not used as a real store event

#define LOG_ERROR     0
#define LOG_INFO      3
#define LOG_DEBUG     5

#include "DorgemEvent.h"

class CCaptionManager;

/////////////////////////////////////////////////////////////////////////////
//

class CStoreEvent : public CDorgemEvent
{
public:
  CStoreEvent();
  virtual ~CStoreEvent();

  virtual bool NeedStore();
  virtual bool Read(TiXmlElement*);
  virtual bool Write(TiXmlElement*);

  // Pure virtual function
  virtual bool Store() = 0;

  BOOL    HasCaptions()           { return(m_pCaptions != NULL);  };
  time_t  GetInterval()           { return(m_tInterval);          };
  wxString GetErrorMsg()          { return(m_strErrorMsg);        };
  //UINT    GetLogLevel()           { return(m_uLogLevel);          };
  //UINT    GetHistory()            { return(m_uHistory);           };
  bool    GetResetTime()          { return(m_bResetTime);         };
  //void    GetInterval(time_t&, int&);
  void    DeleteCaptions();
  CCaptionManager* GetCaptions()  {return(m_pCaptions);           };

  //void SetInterval(time_t t)            { m_tInterval = t;        };
  //void SetLogLevel(UINT u)              { m_uLogLevel = u;        };
  //void SetHistory(UINT u)               { m_uHistory = u;         };
  void SetCaptions(CCaptionManager* p)  { m_pCaptions = p;        };
  void SetImage(wxImage& img)           { m_CurImage = img;       };
  //void SetLogFile(CString, BOOL =FALSE);

  void SetResetTime(bool b)             { m_bResetTime = b;       };

protected:
  //void Log(int, wxString);
  //bool OpenLogFile(bool);
  void MaintainHistory(wxString);
  void ProcessImage();
  virtual bool MoveFileTo(wxString, wxString);
  bool MotionDetection();
  void ResetTimer();

  bool        m_bBusy;              // Is this event busy
  wxDateTime  m_tmLastUpdate;       // When was the file last stored
  wxImage     m_CurImage;           // The image as we get it from the camera
  bool        m_bResetTime;         // Reset time if storage failes
  time_t      m_tInterval;          // What is the interval (in seconds)
  wxString    m_strErrorMsg;        // The last error message
  int         m_iQuality;           // Quality of the JPEG file (0-100: 0 is terrible, 100 is very good)
  int         m_iLogLevel;          // Current level of logging
  int         m_iHistory;           // Number of files to keep as history (File,FTP)
  //wxFileName  m_fnLogFile;          // The name of the log file
  //CStdioFile       m_fileLog;       // Log file

private:
  bool        m_bMotionDetection;   // Are we using motion detection?
  wxFileName  m_fnMDOverlay;        // Overlay to exclude parts from motion detection
  int         m_iMDSensitivity;     // Sensitivity of motion detection
  wxImage     m_PrevImage;          // The previous image we got from the camera (for motion detection)

  int         m_iRotate;            // Rotate the image?
  CCaptionManager* m_pCaptions;     // The captions for this event

};

#endif // __STOREEVENT_H__
