/////////////////////////////////////////////////////////////////////////////
//                                                                         //
//  Dorgem - Webcam Capture application                                    //
//  Copyright (C) 2001-2007 Frank Fesevur                                  //
//                                                                         //
//  This program is free software; you can redistribute it and/or modify   //
//  it under the terms of the GNU General Public License as published by   //
//  the Free Software Foundation; either version 2 of the License, or      //
//  (at your option) any later version.                                    //
//                                                                         //
//  This program is distributed in the hope that it will be useful,        //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of         //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the           //
//  GNU General Public License for more details.                           //
//                                                                         //
//  You should have received a copy of the GNU General Public License      //
//  along with this program; if not, write to the Free Software            //
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.              //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////
// StoreFTP.cpp : implementation file

#include "Dorgem.h"
#include "StoreFTP.h"

#include <wx/protocol/ftp.h>

#ifdef __WXDEBUG__
//#define __NO_FTP__     // Useful for testing without an active Internet connection
#endif

/////////////////////////////////////////////////////////////////////////////
//

CStoreFTP::CStoreFTP()
{
  m_iType = STORE_FTP;
  m_bKeepOpen = false;
  m_pConnection = NULL;
  m_bPassiveFTP = true;
}

/////////////////////////////////////////////////////////////////////////////
//

CStoreFTP::~CStoreFTP()
{
  CloseConnection(true);
}

/////////////////////////////////////////////////////////////////////////////
//

bool CStoreFTP::OpenConnection()
{
  bool bRet = true;

  if (m_pConnection == NULL)
  {
    m_pConnection = new wxFTP;
    wxASSERT(m_pConnection != NULL);
    m_pConnection->SetUser(m_strUser);
    m_pConnection->SetPassword(m_strPassword);
    m_pConnection->SetPassive(m_bPassiveFTP);

    bRet = m_pConnection->Connect(m_strHost);
    if (bRet)
      m_pConnection->SetBinary();
    else
    {
      m_strErrorMsg = m_pConnection->GetLastResult();
      wxLogDebug("FTP Error: %s", m_strErrorMsg);
      //Log(LOG_ERROR, m_strErrorMsg);
      //GetMainWnd()->SetStatusBar("Error connecting to " + m_strHost);
    }
  }

  return(bRet);
}

/////////////////////////////////////////////////////////////////////////////
//

void CStoreFTP::CloseConnection(bool bAlwaysClose)
{
  if (m_bKeepOpen && !bAlwaysClose)
    return;

  // If the connection is open, close it
  if (m_pConnection != NULL)
  {
    m_pConnection->Close();
    delete m_pConnection;
    m_pConnection = NULL;
  }
}

/////////////////////////////////////////////////////////////////////////////
//

bool CStoreFTP::MoveFileTo(wxString strFrom, wxString strTo)
{
  return(m_pConnection->Rename(strFrom, strTo));
}

/////////////////////////////////////////////////////////////////////////////
//

bool CStoreFTP::Store()
{
  wxLogDebug("[CStoreFTP]Store(%s)", m_strName);

  // Is this event enabled?
  if (!m_bEnabled)
  {
    ResetTimer();
    return(true);
  }

  // Is there a valid image available
  if (!m_CurImage.IsOk())
  {
    ResetTimer();
    return(true);
  }

  // Is it already busy?
  if (m_bBusy)
  {
    ResetTimer();
    return(true);
  }
  m_bBusy = true;

  // Is there any motion detected or motion detection disabled?
  if (!MotionDetection())
  {
    ResetTimer();
    m_bBusy = false;
    return(true);
  }

  // These members need to be set
  wxASSERT(!m_strHost.IsEmpty());
  wxASSERT(!m_strUser.IsEmpty());
  wxASSERT(!m_strPassword.IsEmpty());
  wxASSERT(!m_strFile.IsEmpty());

#ifdef __NO_FTP__
  bool bRet = true;  // Pretend we stored OK
#else
  bool bRet = false;

  // Do we need to open the connection?
  if (!OpenConnection())
    return(false);

  if (m_pConnection != NULL)
  {
    // Create the filename that needs to be stored
    wxString strFile = wxGetApp().DateFormat(m_strFile);

    // Do we need to maintain a history?
    MaintainHistory(strFile);

    // Contains changes made by Aaron Kondziela - aaronk@pscomputing.com
    wxString strFileTemp = strFile + wxT(".temp"); // Temporary file name
    wxString strFileTemp2 = strFile + wxT(".old"); // This is for the original file when swapping

    // Upload the image
    wxOutputStream *out = m_pConnection->GetOutputStream(strFileTemp);
    if (out != NULL)
    {
      //
      ProcessImage();

      // Store the image as a JPEG in a memory stream
      wxMemoryOutputStream outStream;
      m_CurImage.SetOption(wxT("quality"), m_iQuality);
      m_CurImage.SaveFile(outStream, wxBITMAP_TYPE_JPEG);

      // Create an input memory stream based upon this stream
      // because that is what FTP needs
      wxMemoryInputStream inStream(outStream);

      // Now we can put the data on the FTP site
      out->Write(inStream);
      delete out;

      // Quickly swap in the new image where the old one was
      // Don't catch errors here because these files may not exist
      MoveFileTo(strFile, strFileTemp2);    // Rename the existing file if any
      MoveFileTo(strFileTemp, strFile);     // Rename the temporary file to the real name
      m_pConnection->RmFile(strFileTemp2);  // Delete the actual filename if it already exists
    }
  }

  CloseConnection(false);

#endif

  if (bRet || m_bResetTime)
    m_tmLastUpdate = wxDateTime::Now();

  if (bRet)
  {
/*
    CString strTmp = "Stored succesfully";
    Log(LOG_INFO, strTmp);
    strTmp += " with FTP on ";
    strTmp += m_strHost;
    GetMainWnd()->SetStatusBar(strTmp);
*/
  }

  m_bBusy = false;
  return(bRet);
}

/////////////////////////////////////////////////////////////////////////////
//

bool CStoreFTP::Read(TiXmlElement* pElemProfile)
{
  if (!CStoreEvent::Read(pElemProfile))
    return(false);

  m_strHost = wxGetApp().GetAttributeString(pElemProfile, g_strXmlHost);
  m_strUser = wxGetApp().GetAttributeString(pElemProfile, g_strXmlUser);
  m_strPassword = wxGetApp().GetAttributeString(pElemProfile, g_strXmlPassword);
  m_strFile = wxGetApp().GetAttributeString(pElemProfile, g_strXmlFile);
  m_bKeepOpen = wxGetApp().GetAttributeBool(pElemProfile, g_strXmlKeepOpen, false);
  m_bPassiveFTP = wxGetApp().GetAttributeBool(pElemProfile, g_strXmlPassive, true);
  //wxGetApp().DeCrypt(m_strPassword);
  return(true);
}

/////////////////////////////////////////////////////////////////////////////
//

bool CStoreFTP::Write(TiXmlElement* pElemProfiles)
{
  TiXmlElement elemProfile(g_strXmlProfile);
  elemProfile.SetAttribute(g_strXmlType, g_strXmlFTP);

  // Add the settings from the base-classes
  CStoreEvent::Write(&elemProfile);

  // Encrypt the password
  wxString strPassword = m_strPassword;
  //wxGetApp().Crypt(strPassword);

  // Add the settings from this class
  elemProfile.SetAttribute(g_strXmlHost, m_strHost);
  elemProfile.SetAttribute(g_strXmlUser, m_strUser);
  elemProfile.SetAttribute(g_strXmlPassword, strPassword);
  elemProfile.SetAttribute(g_strXmlFile, m_strFile);
  elemProfile.SetAttribute(g_strXmlKeepOpen, m_bKeepOpen);
  elemProfile.SetAttribute(g_strXmlKeepOpen, m_bPassiveFTP);

  // Add it to the main list
  pElemProfiles->InsertEndChild(elemProfile);
  return(true);
}
