/////////////////////////////////////////////////////////////////////////////
//                                                                         //
//  Dorgem - Webcam Capture application                                    //
//  Copyright (C) 2001-2007 Frank Fesevur                                  //
//                                                                         //
//  This program is free software; you can redistribute it and/or modify   //
//  it under the terms of the GNU General Public License as published by   //
//  the Free Software Foundation; either version 2 of the License, or      //
//  (at your option) any later version.                                    //
//                                                                         //
//  This program is distributed in the hope that it will be useful,        //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of         //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the           //
//  GNU General Public License for more details.                           //
//                                                                         //
//  You should have received a copy of the GNU General Public License      //
//  along with this program; if not, write to the Free Software            //
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.              //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////
// StoreFile.cpp : implementation file

#include "Dorgem.h"
#include "StoreEvent.h"
#include "StoreFile.h"
#include "DorgemDlg.h"

/////////////////////////////////////////////////////////////////////////////
//

CStoreFile::CStoreFile()
{
  m_iType = STORE_FILE;
  m_bCreateDirs = false;
}

/////////////////////////////////////////////////////////////////////////////
//

bool CStoreFile::MoveFileTo(wxString strFrom, wxString strTo)
{
  return(wxRenameFile(strFrom, strTo));
}

/////////////////////////////////////////////////////////////////////////////
//

bool CStoreFile::Store()
{
  wxLogDebug("[CStoreFile]Store(%s)", m_strName);

  // Is this event enabled?
  if (!m_bEnabled)
    return(true);

  // Is there a valid image available
  if (!m_CurImage.IsOk())
    return(true);

  // Is it already busy?
  if (m_bBusy)
    return(true);
  m_bBusy = true;

  // Is there any motion detected or motion detection disabled?
  if (!MotionDetection())
  {
    ResetTimer();
    m_bBusy = false;
    return(true);
  }

  // 
  ProcessImage();

  wxDateTime tmNow = wxDateTime::Now();
  wxString strFile = tmNow.Format(m_fnStoreFile.GetFullPath());

  // Do we need to create the directories
  if (m_bCreateDirs)
  {
    // Is there a volume? (aka MS-DOS Drive)
    wxString strVolume, strPath;
    wxFileName::SplitVolume(strFile, &strVolume, &strPath);
    if (strVolume.IsEmpty())
      strPath.Empty();
    else
      strPath = strVolume + wxFileName::GetVolumeSeparator();

    // Go through the directories in the path and see
    // if we need to create any of these directories
    wxFileName fn(strFile);
    wxArrayString strDirs = fn.GetDirs();
    for (size_t i = 0; i < strDirs.GetCount(); i++)
    {
      strPath += wxFileName::GetPathSeparator();
      strPath += strDirs[i];
      wxLogDebug("Dir %d = '%s' '%s'", i, strDirs[i], strPath);

      if (!wxDirExists(strPath))
        wxMkdir(strPath);
    }
  }

  // Do we need to maintain a history?
  MaintainHistory(strFile);

  // Save the image
  m_CurImage.SetOption(wxT("quality"), m_iQuality);
  bool bRet = m_CurImage.SaveFile(strFile, wxBITMAP_TYPE_JPEG);

/*
  if (!bRet)
  {
    m_strErrorMsg.Format("Failed to save '%s': %s", strFile,
                                   GetApp()->GetSystemError(GetLastError()));
    TRACE("Error: '%s'\n", m_strErrorMsg);
    Log(LOG_ERROR, m_strErrorMsg);
    GetMainWnd()->SetStatusBar("Error saving image " + strFile);
  }
*/

  if (bRet || m_bResetTime)
    m_tmLastUpdate = wxDateTime::Now();

/*
  if (bRet)
  {
    CString strTmp = "Stored succesfully";
    Log(LOG_INFO, strTmp);
    strTmp += " to file ";
    strTmp += strFile;
    GetMainWnd()->SetStatusBar(strTmp);
  }
*/

  m_bBusy = false;
  return(bRet);
}

/////////////////////////////////////////////////////////////////////////////
//

bool CStoreFile::Read(TiXmlElement* pElemProfile)
{
  if (!CStoreEvent::Read(pElemProfile))
    return(false);

  m_fnStoreFile = wxGetApp().GetAttributeString(pElemProfile, g_strXmlFile);
  m_bCreateDirs = wxGetApp().GetAttributeBool(pElemProfile, g_strXmlCreateDirs, true);
  return(true);
}

/////////////////////////////////////////////////////////////////////////////
//

bool CStoreFile::Write(TiXmlElement* pElemProfiles)
{
  TiXmlElement elemProfile(g_strXmlProfile);
  elemProfile.SetAttribute(g_strXmlType, g_strXmlFile);

  // Add the settings from the base-classes
  CStoreEvent::Write(&elemProfile);

  // Add the setiings from this class
  elemProfile.SetAttribute(g_strXmlFile, m_fnStoreFile.GetFullPath());
  elemProfile.SetAttribute(g_strXmlCreateDirs, m_bCreateDirs);

  // Add it to the main list
  pElemProfiles->InsertEndChild(elemProfile);
  return(true);
}
